
#include <iwlib.h>

#ifdef HAVE_LIBOSSO
#include <libosso.h>
#else
#include <glib.h>
#include <dbus/dbus.h>

#define WIFISCAN_INTROSPECTION "<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n\
         \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n\
<node name=\"/org/javiplx/wifiscan\">\n\
  <interface name=\"org.javiplx.wifiscan\">\n\
    <method name=\"wakeup\">\n\
      <arg name=\"result\" type=\"s\" direction=\"out\" />\n\
    </method>\n\
    <method name=\"start\">\n\
      <arg name=\"result\" type=\"s\" direction=\"out\" />\n\
    </method>\n\
    <method name=\"scan\">\n\
      <arg name=\"result\" type=\"s\" direction=\"out\" />\n\
    </method>\n\
    <method name=\"stop\">\n\
      <arg name=\"result\" type=\"s\" direction=\"out\" />\n\
    </method>\n\
  </interface>\n\
</node>\n"

#define OSSO_OK 0
#define OSSO_ERROR 1

#define osso_context_t DBusConnection
typedef struct osso_rpc_t_values {
  int i;
  char *s;
  } osso_rpc_t_values;
typedef struct osso_rpc_t {
  int type;
  osso_rpc_t_values value;
  } osso_rpc_t;
#endif

#define WIFISCAND_VERSION_STRING  "1.1"

#define OSSO_NAME    "wifiscan"
#define OSSO_SERVICE "org.javiplx."OSSO_NAME
#define OSSO_OBJECT  "/org/javiplx/"OSSO_NAME
#define OSSO_IFACE   "org.javiplx."OSSO_NAME


typedef struct {
	char	      mac[18];
	int	      rssi;
	int	      noise;
} ScanInfo;


/* --------------------------------------------------------------------------- WirelessInterface */
typedef struct {
	wireless_info info;
	char*	      ifname;
	int	      sock;
} WirelessInterface;

/* ------------------------------------------------------------------ WirelessInterface_ScanItem */

void WirelessInterface_ScanItem(struct wireless_scan* ap, ScanInfo **scaninfo) {

			*scaninfo = (ScanInfo*) malloc( sizeof(ScanInfo) );
			memset(*scaninfo, 0, sizeof(ScanInfo) );

			iw_sawap_ntop( &ap->ap_addr, (*scaninfo)->mac);

			(*scaninfo)->rssi = ap->stats.qual.level - 0x100;
			(*scaninfo)->noise = ap->stats.qual.level - 0x100;

}

/* ---------------------------------------------------------------------- */

void* makeScan(WirelessInterface* wlan, gchar *message) {
	wireless_scan_head context;
	iwrange        range;
	int delay = 1;

	ScanInfo	**scan = NULL;

	iw_get_range_info(wlan->sock, wlan->ifname, &range);

	/* We don't call iw_scan to allow fine control of delays and timeouts */
	context.result = NULL;
	context.retry = 0;

	while(delay>0) {
		delay = iw_process_scan(wlan->sock, wlan->ifname, range.we_version_compiled, &context);
		if(delay < 0) break;
		usleep(delay * 1000);
		}

	if ( delay == 0 ) {

		int i = 1; // To acount for the final NULL
		struct wireless_scan *ap = context.result;
		while ( ap != NULL ) {
					ap = ap->next;
					i++;
			}
		scan = (ScanInfo**) malloc( i * sizeof(ScanInfo *) );
		memset(scan, 0, i * sizeof(ScanInfo *) );

		ScanInfo **sc = scan;

		/* Extract values */
		ap = context.result;
		while ( ap != NULL ) {
				WirelessInterface_ScanItem(ap, sc++);
					ap = ap->next;
			}

	} else {
		message = "Unknown error";
	}

	return scan;
}

static void refresh(WirelessInterface* coso) {
	struct iwreq wrq;
	
	iw_get_basic_config(coso->sock, coso->ifname, &(coso->info.b));
	iw_get_range_info(coso->sock, coso->ifname, &(coso->info.range));

	iw_get_ext(coso->sock, coso->ifname, SIOCGIWRATE, &wrq);
	memcpy(&(coso->info.bitrate), &wrq.u.bitrate, sizeof(iwparam));

	iw_get_ext(coso->sock, coso->ifname, SIOCGIWAP, &wrq);
	memcpy(&(coso->info.ap_addr), &wrq.u.ap_addr, sizeof (sockaddr));

	iw_get_stats(
		coso->sock, coso->ifname, &(coso->info.stats), 
		&(coso->info.range), coso->info.has_range
	);
}

/* Application UI data struct */
typedef struct _AppData AppData;
struct _AppData {
    WirelessInterface iface;
    osso_context_t *osso_context;
};

#ifdef HAVE_LIBOSSO
static GMainLoop *event_loop = NULL;
#endif
static short int start_flags = 0;

/* Callback for normal D-BUS messages */
gint dbus_req_handler(const gchar * interface, const gchar * method,
                      GArray * arguments, gpointer data,
                      osso_rpc_t * retval)
{
    AppData *appdata;
    appdata = (AppData *) data;

    retval->type = DBUS_TYPE_STRING;
    retval->value.s = (gchar*) malloc( sizeof(gchar *) );
    retval->value.s[0] = '\0';

#ifndef HAVE_LIBOSSO
    if ( strcmp(method,"Introspect")==0 ) {
        retval->value.s = (gchar *) realloc(retval->value.s,630*sizeof(gchar *));
        snprintf(retval->value.s,strlen(WIFISCAN_INTROSPECTION),WIFISCAN_INTROSPECTION);
        return OSSO_OK;
    }
#endif

    if ( strcmp(method,"wakeup")==0 ) {
        retval->value.s = (gchar *) realloc(retval->value.s,16*sizeof(gchar *));
        snprintf(retval->value.s,16,"WifiScand ready");
        return OSSO_OK;
    }

    if ( strcmp(method,"start")==0 ) {

        if( (appdata->iface.sock=iw_sockets_open()) < 0) {
            retval->value.s = (gchar *) realloc(retval->value.s,33*sizeof(gchar *));
            snprintf(retval->value.s,33,"Failure in socket initialization");
            return OSSO_ERROR;
        }

        /* Get range stuff */
	iwrange        range;
        int has_range = (iw_get_range_info(appdata->iface.sock, appdata->iface.ifname, &range) >= 0);

        /* Check if the interface could support scanning. */
        if((!has_range) || (range.we_version_compiled < 14)) {
            retval->value.s = (gchar *) realloc(retval->value.s,35*sizeof(gchar *));
            snprintf(retval->value.s,35,"Interface doesn't support scanning");
            return OSSO_ERROR;
        }

        struct ifreq frq;
        strncpy(frq.ifr_name, appdata->iface.ifname, IFNAMSIZ);
        if(ioctl(appdata->iface.sock, SIOCGIFFLAGS, &frq)) {
            retval->value.s = (gchar *) realloc(retval->value.s,28*sizeof(gchar *));
            snprintf(retval->value.s,28,"Cannot get interface status");
            return OSSO_ERROR;
        }

        start_flags = frq.ifr_flags;
        frq.ifr_flags |= IFF_UP | IFF_RUNNING;

        if(ioctl(appdata->iface.sock, SIOCSIFFLAGS, &frq)) {
            retval->value.s = (gchar *) realloc(retval->value.s,27*sizeof(gchar *));
            snprintf(retval->value.s,27,"Cannot set interface state");
            return OSSO_ERROR;
        }

        refresh(&appdata->iface);
        retval->value.s = (gchar *) realloc(retval->value.s,22*sizeof(gchar *));
        snprintf(retval->value.s,22,"Interface initialized");
        return OSSO_OK;
    }

    if ( strcmp(method,"stop")==0 ) {
	struct ifreq frq;
	strncpy(frq.ifr_name, appdata->iface.ifname, IFNAMSIZ);
	if(!ioctl(appdata->iface.sock, SIOCGIFFLAGS, &frq)) {
	    frq.ifr_flags = start_flags;
	    if(!ioctl(appdata->iface.sock, SIOCSIFFLAGS, &frq))
                refresh(&appdata->iface);
	}
        iw_sockets_close(appdata->iface.sock);
	appdata->iface.sock = 0;
#ifdef HAVE_LIBOSSO
        osso_deinitialize(appdata->osso_context);
        /* Instead of exiting, signaling finish to main loop could be better
        retval->value.s = (gchar *) realloc(retval->value.s,34*sizeof(gchar *));
        snprintf(retval->value.s,34,"Interface moved to original state");
        */
        exit(0);
#else
        return OSSO_OK;
#endif
    }

    if ( strcmp(method,"scan")==0 ) {

        ScanInfo **scan = (ScanInfo **) makeScan(&appdata->iface,retval->value.s);
        if(scan == NULL) {
            retval->value.s = (gchar *) realloc(retval->value.s,64*sizeof(gchar *));
            snprintf(retval->value.s,64,"ERROR");
            return OSSO_ERROR;
        }

        int i = 0;
        while ( *(scan+i) != NULL ) i++;
        retval->value.s = (gchar *) malloc( (22*i+1) * sizeof(gchar *) );
	    if ( retval->value.s == NULL ) {
                retval->value.s = "Error allocating memory";
                return OSSO_ERROR;
	        }
        memset(retval->value.s, '\0', (22*i+1) * sizeof(gchar *) );
        i = 0;
        while ( *scan != NULL ) {
            sprintf(retval->value.s+(22*i),"%s:%d ",(*scan)->mac,(*scan)->rssi);
            scan++;
            i++;
        }

        retval->value.s[strlen(retval->value.s)-1]  = '\0';
        return OSSO_OK;
    }

    retval->value.s = (gchar *) realloc(retval->value.s,64*sizeof(gchar *));
    snprintf(retval->value.s,64,"Unknown method");
    return OSSO_ERROR;
}

#ifndef HAVE_LIBOSSO

dbus_bool_t stopped_service = FALSE;

static DBusObjectPathVTable *vtable = NULL;

int dbus_set_cb_f( DBusConnection *context,
		const char *service, const char *object,
		const char *interface, void *handler, // FIXME : Set proper type for handler
		void *user_data) {

	/* First, we prepare the complex dbus handler structures */
	DBusObjectPathVTable *vtable = malloc( sizeof(DBusObjectPathVTable) );
	memset(vtable , '\0', sizeof(DBusObjectPathVTable) );
	vtable->message_function = (DBusObjectPathMessageFunction) handler; // FIXME : Aqui va el nuevo handler/wrapper

	if (!dbus_connection_register_fallback(context, object, vtable, user_data)) {
		return OSSO_ERROR;
		}

	DBusError error;
	dbus_error_init(&error);

	dbus_bus_request_name (context, service, 0, &error);
	if ( dbus_error_is_set(&error) ) {
		dbus_error_free (&error);
		return OSSO_ERROR;
		}

	dbus_error_free (&error);
	return OSSO_OK;
}

void dbus_deinitialize( DBusConnection *context ) {
	free(vtable); vtable = NULL;
	dbus_connection_unref(context);
	dbus_shutdown();
}

static DBusHandlerResult handler_wrapper (DBusConnection  *connection,
                   DBusMessage     *message,
                   void            *data) {

	gint retcode;
	osso_rpc_t *retval = malloc(sizeof(osso_rpc_t));

	if ( dbus_message_get_type(message) == DBUS_MESSAGE_TYPE_METHOD_CALL ) {
		retcode = dbus_req_handler(dbus_message_get_interface(message),
				dbus_message_get_member(message),
				NULL, data, retval);

		if ( strcmp(dbus_message_get_member(message),"stop")==0 )
			stopped_service = TRUE;

		if ( retval->value.s != NULL ) {
			DBusMessage *reply = dbus_message_new_method_return (message);
			if (reply == NULL)
				exit(0);
			if (!dbus_message_append_args (reply, DBUS_TYPE_STRING, &retval->value.s, DBUS_TYPE_INVALID))
				exit(0);
			if (!dbus_connection_send (connection, reply, NULL))
				exit(0);
			dbus_message_unref (reply);
			}

		}

	return DBUS_HANDLER_RESULT_HANDLED;
}

#endif

int main( void ) {

	osso_context_t *osso_context;
#ifndef HAVE_LIBOSSO
	DBusError error;
#endif

	/* Initialize maemo application */
#ifdef HAVE_LIBOSSO
	osso_context = osso_initialize(OSSO_NAME, WIFISCAND_VERSION_STRING, TRUE, NULL);
#else
	dbus_error_init(&error);
	osso_context = dbus_bus_get(DBUS_BUS_SESSION, &error);
#endif

	/* Check that initialization was ok */
	if (osso_context == NULL) {
#ifndef HAVE_LIBOSSO
		fprintf (stderr, "*** Failed to open connection to activating message bus: %s\n", error.message);
		dbus_error_free (&error);
#endif
		exit(OSSO_ERROR);
		}

	/* Create AppData */
	AppData *appdata;
	appdata = g_new0(AppData, 1);
	appdata->osso_context = osso_context;

	memset(&(appdata->iface.info), 0, sizeof(wireless_info));
	appdata->iface.ifname = "wlan0";
	appdata->iface.sock   = 0;

	/* Add handler for hello D-BUS messages */
#ifdef HAVE_LIBOSSO
	osso_return_t result = osso_rpc_set_cb_f(osso_context, OSSO_SERVICE, OSSO_OBJECT, OSSO_IFACE, dbus_req_handler, appdata);
#else
	int result = dbus_set_cb_f(osso_context, OSSO_SERVICE, OSSO_OBJECT, OSSO_IFACE, handler_wrapper, appdata);
#endif

	if (result != OSSO_OK) {
#ifdef HAVE_LIBOSSO
		osso_system_note_infoprint(appdata->osso_context, "Failure while setting OSSO callback", NULL);
#endif
		return OSSO_ERROR;
	}

	/* INITIALIZATION FINISH */

#ifdef HAVE_LIBOSSO
	event_loop = g_main_loop_new(NULL, FALSE);
	g_main_loop_run(event_loop);
#else
	while (dbus_connection_read_write_dispatch(osso_context, -1) && stopped_service==FALSE) {}
#endif

	/* Deinitialize OSSO */
#ifdef HAVE_LIBOSSO
	osso_deinitialize(osso_context);
#else
	dbus_deinitialize(osso_context);
#endif

	exit(0);
}

